@@ -685,4 +685,59 @@ def _neighbor(
             constant_matrix,
             tracer
             )
+def _tree2nwk(
+        tree,
+        taxa,
+        distances
+        ):
+    """
+    Convert the tree-matrix created by the _upgma-function to newick representation.
+    
+    Parameters
+    ----------
+    tree_matrix : list
+        The tree-representation that is yielded by _upgma, also used in
+        scipy-cluster algorithms.
+    taxa : list 
+        List of the taxa (or sequences) in the order in which the tree was
+        created.
+    distances : bool
+        Specify whether distances should be included in the string or not.
 
+    Returns
+    -------
+    newick : str
+        A newick string.
+
+    """
+
+#     cdef int i,a,b
+#     cdef float c,d
+    x = len(taxa)
+#     cdef str newick_string
+
+    newick = dict([(i,taxa[i]) for i in range(x)])
+    
+    # create different output, depending on the options for the inclusion of
+    # distances or topology only
+    if distances:
+        for i,(a,b,c,d) in enumerate(tree):
+            newick[x+i] = '({0}:{2:.2f},{1}:{3:.2f})'.format(
+                    newick[a],
+                    newick[b],
+                    c,
+                    d
+                    )
+    else:
+        for i in range(len(tree)):
+            newick[x+i] = '({0},{1})'.format(
+                    newick[tree[i][0]],
+                    newick[tree[i][1]]
+                    )
+    
+    newick_string = newick[max(newick.keys())] + ';'
+
+    return newick_string    
+    
+    
+    
