@@ -1,30 +1,23 @@
 # author   : Johann-Mattis List
 # email    : mattis.list@gmail.com
 # created  : 2013-03-06 16:41
-# modified : 2013-06-26 17:40
+# modified : 2013-11-18 09:36
 """
 Module provides classes and functions for multiple alignment analyses.
 """
 
 __author__="Johann-Mattis List"
-__date__="2013-06-26"
+__date__="2013-11-18"
 
 # thirdparty imports
 import numpy as np
 
-# lingpy imports
-#from ..data import *
-try:
-    from ..algorithm.cython import calign
-    from ..algorithm.cython import talign
-    from ..algorithm.cython import cluster
-    from ..algorithm.cython import misc
-except:
-    from ..algorithm.cython import _calign as calign
-    from ..algorithm.cython import _talign as talign
-    from ..algorithm.cython import _cluster as cluster
-    from ..algorithm.cython import _misc as misc
+from ..algorithm import calign
+from ..algorithm import talign
+from ..algorithm import cluster
+from ..algorithm import misc
 
+from ..thirdparty.cogent import LoadTree
 from ..sequence.sound_classes import *
 from ..settings import rcParams
 
@@ -65,6 +58,7 @@ class Multiple(object):
                 "breaks"       : rcParams['breaks'],
                 "stress"       : rcParams["stress"],
                 "merge_vowels" : rcParams["merge_vowels"],
+                "unique_seqs"  : rcParams["unique_sequences"]
                 }
         kw.update(keywords)
 
@@ -91,15 +85,21 @@ class Multiple(object):
                         [str(i+1)+'.'+str(j+1) for j in range(len(tokens))]
                         )
 
-        # create dictionary of all unique sequences, this is important, since
-        # identical sequences should only be counted once in an alignment,
-        # since they otherwise may disturb the analysis or slow it down
+
         self.uniseqs = {}
-        for i,seq in enumerate(self.seqs):
-            try:
-                self.uniseqs[seq] += [i]
-            except:
-                self.uniseqs[seq] = [i]
+        self.unique_seqs = kw["unique_seqs"]
+        if self.unique_seqs:
+            # create dictionary of all unique sequences, this is important, since
+            # identical sequences should only be counted once in an alignment,
+            # since they otherwise may disturb the analysis or slow it down
+            for i,seq in enumerate(self.seqs):
+                try:
+                    self.uniseqs[seq] += [i]
+                except:
+                    self.uniseqs[seq] = [i]
+        else:  
+            #no uniqueness filtering      
+            self.uniseqs = range(0,len(self.seqs))
 
         self._length = len(self.uniseqs)
         
@@ -234,7 +234,7 @@ class Multiple(object):
             model = None,
             classes = True,
             sonar = True,
-            scorer = {}
+            scoredict = {}
             ):
         """
         Method defines a specific class model for the calculation.
@@ -244,13 +244,17 @@ class Multiple(object):
         model : { None ~lingpy.data.model.Model } (default=None)
             A sound class model.
         """
+        # check whether model is a string
+        if not hasattr(model,'name'):
+            model = rcParams[model]
+
         # check for keyword classes
         if not classes:
             classify = lambda x:x
         else: 
             # check for the model
             if not model:
-                self.model = sca
+                self.model = rcParams['sca']
             else:
                 self.model = model
 
@@ -273,10 +277,13 @@ class Multiple(object):
                 indices[tuple(seq)] += [i]
             except:
                 indices[tuple(seq)] = [i]
-
+                
         # create additional matrices for the internal representation of the
         # class sequences
-        keys = [val[0] for val in indices.values()]
+        if self.unique_seqs:
+            keys = [val[0] for val in indices.values()]
+        else:
+            keys = range(len(self.classes))  
         self.height = len(keys)
 
         # add the classes
@@ -284,9 +291,30 @@ class Multiple(object):
         self._numbers = [[str(i+1)+'.'+str(j+1) for j in
             range(len(self._classes[i]))] for i in range(self.height)]
 
+        # create an index which allows to quickly interchange between classes
+        # and given sequences (trivial without sequence uniqueness
+        if self.unique_seqs:
+            self.int2ext = dict(
+                    [(i,indices[tuple(self._classes[i])]) for i in range(len(keys))]      
+                    )
+        else:
+            self.int2ext = dict(  
+                    [(i,[i]) for i in range(len(keys))]    
+                    )
+
+        # -> # create external to internal in order to allow for a quick switching
+        # -> # of the vals
+        # -> self.ext2int = {}
+        # -> for k,vals in self.int2ext.items():
+        # ->     for v in vals:
+        # ->         self.ext2int[v] = k
+
         # store sonars if they are passed as a list
         if type(sonar) == list:
-            self._sonars = sonar
+            self._sonars = [sonar[key] for key in keys]
+            # -> self._sonars = [0 for i in range(len(sonar))]
+            # -> for i in range(len(self._sonars)):
+            # ->     self._sonars[self.ext2int[i]] = sonar[i]
             self._prostrings = list([prosodic_string(s) for s in self._sonars])
         # create sonars if the argument is true
         elif sonar:
@@ -305,20 +333,14 @@ class Multiple(object):
         else:
             self._sonars = False
             self._prostrings = False
-
-        # create an index which allows to quickly interchange between classes
-        # and given sequences
-        self.int2ext = dict(
-                [(i,indices[tuple(self._classes[i])]) for i in range(len(keys))]    
-                )
         
         # create a scoredict for the calculation of alignment analyses
         # append the scorer if it is given with the model
         if classes:
             scorer = lambda x,y:self.model.scorer[x,y]
         # leave the scorer that was passed if it is not empty
-        elif scorer:
-            scorer = lambda x,y:scorer[x,y]
+        elif scoredict:
+            scorer = lambda x,y:scoredict[x,y]
         # create a short function if the scorer is empty
         else:
             def scorer(x,y):
@@ -384,11 +406,14 @@ class Multiple(object):
             gop = -2,
             scale = 0.5, 
             factor = 0.3,
-            restricted_chars = 'T_'
+            restricted_chars = 'T_',
+            **keywords
             ):
         """
         Function calculates all pairwise alignments from the data.
         """
+        if 'transform' not in keywords:
+            keywords['transform'] = rcParams['align_transform']
         
         # create array for alignments
         self._alignments = [[0 for i in range(self.height)] for i in range(self.height)]
@@ -399,9 +424,13 @@ class Multiple(object):
         # check for the mode, if sonority profiles are not chose, take the
         # simple alignment function
         if self._sonars:
+
+            # define a lambda-shortcut for creation of prosodic weights
+            make_pro_weights = lambda x: prosodic_weights(x,_transform = keywords['transform'])
+            
             # get the weights
             if not hasattr(self,'weights'):
-                self._weights = list(map(prosodic_weights,self._prostrings))
+                self._weights = list(map(make_pro_weights,self._prostrings))
             
             alignments = calign.align_pairwise(
                     self._numbers,
@@ -576,6 +605,15 @@ class Multiple(object):
         else:
             raise ValueError('[i] Method <'+tree_calc+'> for tree calculation not available.')
 
+        # create a newick-representation of the string
+        self.tree = LoadTree(
+                cluster._tree2nwk(
+                    self.tree_matrix,
+                    [''.join(c) for c in self._classes],
+                    False
+                    )
+                )
+
     def _align_profile(
             self,
             almsA,
@@ -855,7 +893,7 @@ class Multiple(object):
         Parameters
         ----------
 
-        model : { 'dolgo', 'sca', 'asjp' }
+        model : { "dolgo", "sca", "asjp" } (defaul="sca")
             A string indicating the name of the :py:class:`Model \
             <lingpy.data.model>` object that shall be used for the analysis.
             Currently, three models are supported:
@@ -870,7 +908,7 @@ class Multiple(object):
               of :evobib:`Brown2011` (see the description in
               :evobib:`List2012`.
         
-        mode : { 'global', 'dialign' }
+        mode : { "global", "dialign" } (default="global")
             A string indicating which kind of alignment analysis should be
             carried out during the progressive phase. Select between: 
             
@@ -893,11 +931,16 @@ class Multiple(object):
             The factor by which the initial and the descending position shall
             be modified.
 
-        tree_calc : { 'neighbor', 'upgma' } (default='upgma')
+        tree_calc : { "neighbor", "upgma" } (default="upgma")
             The cluster algorithm which shall be used for the calculation of
             the guide tree. Select between ``neighbor``, the Neighbor-Joining
             algorithm (:evobib:`Saitou1987`), and ``upgma``, the UPGMA
             algorithm (:evobib:`Sokal1958`).
+            
+        guide_tree : tree_matrix
+            Use a custom guide tree instead of performing a cluster algorithm
+            for constructing one based on the input similarities. The use of this
+            option makes the tree_calc option irrelevant. 
 
         gap_weight : float (default=0)
             The factor by which gaps in aligned columns contribute to the
@@ -907,9 +950,10 @@ class Multiple(object):
         
         restricted_chars : string (default="T")
             Define which characters of the prosodic string of a sequence
-            reflect its secondary structure (cf. :evobib:`List2012b`) and should therefore be aligned
-            specifically. This defaults to "T", since this is the character
-            that represents tones in the prosodic strings of sequences.
+            reflect its secondary structure (cf. :evobib:`List2012b`) and
+            should therefore be aligned specifically. This defaults to "T",
+            since this is the character that represents tones in the prosodic
+            strings of sequences.
 
         """
         # set up the defaults parameters stored in the kw dictionary
@@ -922,7 +966,7 @@ class Multiple(object):
                 restricted_chars = rcParams['restricted_chars'],
                 classes          = rcParams['align_classes'],
                 sonar            = rcParams['align_sonar'],
-                scorer           = rcParams['align_scorer'],
+                scoredict           = rcParams['align_scorer'],
                 gop              = rcParams['align_gop'],
                 gap_weight       = rcParams['align_gap_weight']
                 )
@@ -946,10 +990,11 @@ class Multiple(object):
                 )
         
         # set the model
-        self._set_model(model,
+        self._set_model(
+                model,
                 kw['classes'],
                 kw['sonar'],
-                kw['scorer']
+                kw['scoredict']
                 )
 
         # set the scorer
@@ -963,8 +1008,11 @@ class Multiple(object):
                 restricted_chars=kw['restricted_chars']
                 )
 
-        # get the guide-tree 
-        self._make_guide_tree(tree_calc=kw['tree_calc'])
+        # construct or set the guide-tree 
+        if 'guide_tree' in kw.keys():
+            self.tree_matrix = kw['guide_tree']
+        else:
+            self._make_guide_tree(tree_calc=kw['tree_calc'])
 
         # merge the alignments
         self._merge_alignments(
@@ -985,7 +1033,9 @@ class Multiple(object):
             ):
         """
         Carry out a library-based progressive alignment analysis of the sequences.
-
+        
+        Notes
+        -----
         In contrast to traditional progressive multiple sequence alignment
         approaches such as :evobib:`Feng1981` and :evobib:`Thompson1994`,
         library-based progressive alignment :evobib:`Notredame2000` is based on
@@ -998,7 +1048,7 @@ class Multiple(object):
         Parameters
         ----------
 
-        model : { 'dolgo', 'sca', 'asjp' }
+        model : { "dolgo", "sca", "asjp" } (default="sca")
             A string indicating the name of the :py:class:`Model \
             <lingpy.data.model>` object that shall be used for the analysis.
             Currently, three models are supported:
@@ -1013,7 +1063,7 @@ class Multiple(object):
               of :evobib:`Brown2011` (see the description in
               :evobib:`List2012`.
 
-        mode : { 'global', 'dialign' }
+        mode : { "global", "dialign" } (default="global")
             A string indicating which kind of alignment analysis should be
             carried out during the progressive phase. Select between: 
             
@@ -1023,7 +1073,7 @@ class Multiple(object):
             * "dialign" -- global alignment analysis which seeks to maximize
               local similarities :evobib:`Morgenstern1996`.
 
-        modes : list (default=[('global',-10,0.6),('local',-1,0.6)])
+        modes : list (default=[("global",-10,0.6),("local",-1,0.6)])
             Indicate the mode, the gap opening penalties (GOP), and the gap extension scale
             (GEP scale), of the pairwise alignment analyses which
             are used to create the library.
@@ -1041,11 +1091,16 @@ class Multiple(object):
             The factor by which the initial and the descending position shall
             be modified.
 
-        tree_calc : { 'neighbor', 'upgma' }
+        tree_calc : { "neighbor", "upgma" } (default="upgma")
             The cluster algorithm which shall be used for the calculation of
             the guide tree. Select between ``neighbor``, the Neighbor-Joining
             algorithm (:evobib:`Saitou1987`), and ``upgma``, the UPGMA
             algorithm (:evobib:`Sokal1958`).
+            
+        guide_tree : tree_matrix 
+            Use a custom guide tree instead of performing a cluster algorithm
+            for constructing one based on the input similarities. The use of this
+            option makes the tree_calc option irrelevant.
 
         gap_weight : float (default=0)
             The factor by which gaps in aligned columns contribute to the
@@ -1133,9 +1188,12 @@ class Multiple(object):
                 kw['factor'],
                 kw['restricted_chars']
                 )
-
-        # reconstruct the tree
-        self._make_guide_tree(kw['tree_calc'])
+        
+        # construct or set the guide-tree 
+        if 'guide_tree' in kw.keys():
+            self.tree_matrix = kw['guide_tree']
+        else:
+            self._make_guide_tree(tree_calc=kw['tree_calc'])
         
         # merge the alignments, not that the scale doesn't really influence any
         # of the results here, since gap scores are set to 0, gapping should be
@@ -1292,11 +1350,11 @@ class Multiple(object):
         Parameters
         ----------
 
-        alm_matrix : { 'self', 'other' }
+        alm_matrix : { "self", "other" } (default="self")
             Indicate for which MSA the sum-of-pairs score shall be calculated.
 
-        mat : None or :py:class:`numpy.array`
-            If 'other' is chosen as an option for **alm_matrix**, define for
+        mat : { None, list }
+            If "other" is chosen as an option for **alm_matrix**, define for
             which matrix the sum-of-pairs score shall be calculated.
 
         gap_weight : float (default=0)
@@ -1394,17 +1452,17 @@ class Multiple(object):
             iteration ("immediate") or after all iterations have been carried
             out ("final").
          
-        mode : { 'global', 'overlap', 'dialign' }
+        mode : { "global", "overlap", "dialign" } (default="global")
             A string indicating which kind of alignment analysis should be
             carried out during the progressive phase. Select between: 
             
-            * 'global' -- traditional global alignment analysis based on the
+            * "global" -- traditional global alignment analysis based on the
               Needleman-Wunsch algorithm :evobib:`Needleman1970`,
 
-            * 'dialign' -- global alignment analysis which seeks to maximize
+            * "dialign" -- global alignment analysis which seeks to maximize
               local similarities :evobib:`Morgenstern1996`.
 
-            * 'overlap' -- semi-global alignment, where gaps introduced in the
+            * "overlap" -- semi-global alignment, where gaps introduced in the
               beginning and the end of a sequence do not score.     
         
         gop : int (default=-5)
@@ -1471,7 +1529,9 @@ class Multiple(object):
         """
         Iterative refinement based on a flat cluster analysis of the data.
         
-        This method uses the :py:func:`lingpy.algorithm.cluster.flat_upgma`
+        Notes
+        -----
+        This method uses the :py:func:`lingpy.algorithm.clustering.flat_upgma`
         function in order to retrieve a flat cluster of the data.
 
         Parameters
@@ -1485,7 +1545,7 @@ class Multiple(object):
             iteration ("immediate") or after all iterations have been carried
             out ("final").
          
-        mode : { 'global', 'overlap', 'dialign' }
+        mode : { "global", "overlap", "dialign" } (default="global")
             A string indicating which kind of alignment analysis should be
             carried out during the progressive phase. Select between: 
             
@@ -1505,7 +1565,7 @@ class Multiple(object):
             The factor by which the penalty for the extension of gaps (gap
             extension penalty, GEP) shall be decreased. This approach is
             essentially inspired by the exension of the basic alignment
-            algorithm for affine gap penalties [Goto81]_.
+            algorithm for affine gap penalties :evobib:`Gotoh1981`.
 
         factor : float (default=0.3)
             The factor by which the initial and the descending position shall
@@ -1562,19 +1622,21 @@ class Multiple(object):
             ):
         """
         Iterative refinement based on the *Similar Gap Sites* heuristic.
-
+        
+        Notes
+        -----
         This heuristic is fairly simple. The idea is to try to split a given
         MSA into partitions with identical gap sites.
         
         Parameters
         ----------
 
-        check : { 'final', 'immediate' }
+        check : { "final", "immediate" } (default="final")
             Specify when to check for improved sum-of-pairs scores: After each
             iteration ("immediate") or after all iterations have been carried
             out ("final").
          
-        mode : { 'global', 'overlap', 'dialign' }
+        mode : { "global", "overlap", "dialign" } (default="global")
             A string indicating which kind of alignment analysis should be
             carried out during the progressive phase. Select between: 
             
@@ -1630,40 +1692,42 @@ class Multiple(object):
 
     def iterate_all_sequences(
             self,
-            check = 'final',
-            mode = 'global',
+            check = "final",
+            mode = "global",
             gop = -3,
             scale = 0.5,
             factor = 0,
             gap_weight = 1,
-            restricted_chars = 'T_'
+            restricted_chars = "T_"
             ):
         """
         Iterative refinement based on a complete realignment of all sequences.
-
+        
+        Notes
+        -----
         This method essentially follows the iterative method of
-        :evobib:`Barton1987` with
-        the exception that an MSA has already been calculated.
+        :evobib:`Barton1987` with the exception that an MSA has already been
+        calculated.
         
         Parameters
         ----------
 
-        check : { 'final', 'immediate' }
+        check : { "final", "immediate" } (default="final")
             Specify when to check for improved sum-of-pairs scores: After each
             iteration ("immediate") or after all iterations have been carried
             out ("final").
 
-        mode : { 'global', 'overlap', 'dialign' }
+        mode : { "global", "overlap", "dialign" } (default="global")
             A string indicating which kind of alignment analysis should be
             carried out during the progressive phase. Select between: 
             
-            * 'global' -- traditional global alignment analysis based on the
+            * "global" -- traditional global alignment analysis based on the
               Needleman-Wunsch algorithm :evobib:`Needleman1970`,
 
-            * 'dialign' -- global alignment analysis which seeks to maximize
+            * "dialign" -- global alignment analysis which seeks to maximize
               local similarities :evobib:`Morgenstern1996`.
 
-            * 'overlap' -- semi-global alignment, where gaps introduced in the
+            * "overlap" -- semi-global alignment, where gaps introduced in the
               beginning and the end of a sequence do not score.
         
         gop : int (default=-5)
@@ -1673,7 +1737,7 @@ class Multiple(object):
             The factor by which the penalty for the extension of gaps (gap
             extension penalty, GEP) shall be decreased. This approach is
             essentially inspired by the exension of the basic alignment
-            algorithm for affine gap penalties [Goto81]_.
+            algorithm for affine gap penalties :evobib:`Gotoh1981`.
 
         factor : float (default=0.3)
             The factor by which the initial and the descending position shall
@@ -1752,7 +1816,17 @@ class Multiple(object):
             threshold = 2,
             gap_weight = 0.0
             ):
+        """
+        Return all peaks in a given alignment.
 
+        Parameters
+        ----------
+        threshold : { int, float } (default=2)
+            The threshold to determine whether a given column is a peak or not.
+        gap_weight : float (default=0.0)
+            The weight for gaps. 
+
+        """
         peaks = self.get_peaks(gap_weight = gap_weight)
         
 
@@ -1761,16 +1835,6 @@ class Multiple(object):
     def get_pairwise_alignments(
             self,
             **keywords
-            #new_calc = True,
-            #model = None,
-            #mode = 'global',
-            #gop = -3,
-            #scale = 0.5,
-            #factor = 1,
-            #restricted_chars = 'T_',
-            #classes = True,
-            #sonar = True,
-            #scorer = {}
             ):
         """
         Function creates a dictionary of all pairwise alignments  scores.
@@ -1820,12 +1884,6 @@ class Multiple(object):
             The factor by which the initial and the descending position shall
             be modified.
 
-        tree_calc : { 'neighbor', 'upgma' }
-            The cluster algorithm which shall be used for the calculation of
-            the guide tree. Select between ``neighbor``, the Neighbor-Joining
-            algorithm (:evobib:`Saitou1987`), and ``upgma``, the UPGMA
-            algorithm (:evobib:`Sokal1958`).
-
         gap_weight : float (default=0)
             The factor by which gaps in aligned columns contribute to the
             calculation of the column score. When set to 0, gaps will be
@@ -1956,7 +2014,7 @@ class Multiple(object):
         
         Parameters
         ----------
-        mode : { 1, 2, 3, 4, 5 }
+        mode : { 1, 2, 3, 4, 5 } (default=1)
             Indicate which of the four possible PID scores described in :evobib:`Raghava2006`
             should be calculated, the fifth possibility is added for linguistic
             purposes:
@@ -1981,7 +2039,7 @@ class Multiple(object):
 
         See also
         --------
-        lingpy.algorithm.misc.pid
+        lingpy.sequence.sound_classes.pid
 
         """
 
@@ -2208,10 +2266,10 @@ class Multiple(object):
 
         Parameters
         ----------
-        swap_penalty : int or float
+        swap_penalty : { int, float } (default=-3)
             Specify the penalty for swaps in the alignment.
 
-        score_mode : { 'classes', 'library' }
+        score_mode : { "classes", "library" } (default="classes")
             Define the score-mode of the calculation which is either based on
             sound classes proper, or on the specific scores derived from the
             library approach.
@@ -2229,16 +2287,27 @@ class Multiple(object):
 
         Examples
         --------
-        Load a file containing swaps from the test data. 
+        Define a set of strings whose alignment contans a swap. 
 
         >>> from lingpy import *
-        >>> mult = Multiple(get_file('test.msq'))
+        >>> mult = Multiple(["woldemort", "waldemar", "wladimir"])
 
         Align the data, using the progressive approach.
 
         >>> mult.prog_align()
 
         Check for swaps.
+
+        >>> mult.swap_check()
+        True
+
+        Print the alignment
+
+        >>> print(mult)
+        w   o   l   -   d   e   m   o   r   t
+        w   a   l   -   d   e   m   a   r   -
+        v   -   l   a   d   i   m   i   r   -
+
         """
 
         self._set_scorer(score_mode)
@@ -2301,6 +2370,14 @@ def mult_align(
         A two-dimensional list in which alignments are represented as a list of
         tokens.
 
+    Examples
+    --------
+    >>> m = mult_align(["woldemort", "waldemar", "vladimir"], pprint=True)
+    w   o   l   -   d   e   m   o   r   t
+    w   a   l   -   d   e   m   a   r   -
+    -   v   l   a   d   i   m   i   r   -
+    
+
     """
 
     m = Multiple(seqs)
