@@ -15,4 +15,5 @@ __date__="2013-07-12"
 
 # add imorts for basic classes
 from .wordlist import Wordlist
-from .spreadsheet import Spreadsheet
+from .dictionary import Dictionary
+
