@@ -1,7 +1,7 @@
 # author   : Johann-Mattis List
 # email    : mattis.list@gmail.com
 # created  : 2013-03-08 17:30
-# modified : 2013-03-08 17:30
+# modified : 2013-11-21 12:48
 """
 Module for the derivation of sound class models.
 
@@ -11,22 +11,25 @@ access when loading the library, the models are compiled and stored in binary
 files.
 """
 __author__="Johann-Mattis List"
-__date__="2013-03-08"
+__date__="2013-11-21"
+
+
 
-import networkx as nx
 from pickle import dump
 import os
 import codecs
 
 # lingpy imports
 from ..settings import rcParams
-try:
-    from ..algorithm.cython import misc
-except:
-    from ..algorithm.cython import _misc as misc
-from ..convert import *
+from ..algorithm import misc
+from ..convert.strings import scorer2str
 from ..read import *
 
+try:
+    import networkx as nx
+except ImportError:
+    print(rcParams['W_missing_module'].format("networkx"))
+
 def _import_sound_classes(filename):
     """
     Function imports individually defined sound classes from a text file and 
@@ -373,7 +376,7 @@ def _export_score_dict(score_dict):
     """
     
     letters = list(set([key[0] for key in score_dict.keys()]))
-    outfile = codes.open('score_dict.csv','w','utf-8')
+    outfile = codecs.open('score_dict.csv','w','utf-8')
     outfile.write('\t'+'\t'.join(letters)+'\n')
     for letter1 in letters:
         outfile.write(letter1)
@@ -474,7 +477,7 @@ def compile_model(
     See also
     --------
     lingpy.data.model.Model
-    lingpy.data.derive.compile_diacritcs_and_vowels
+    compile_dvt
 
     """
      
@@ -553,14 +556,15 @@ def compile_dvt(path=''):
 
     Notes
     -----
-    Diacritics and vowels are defined in the :file:`data/models/dv/` directory
+    Diacritics, vowels, and tones are defined in the :file:`data/models/dv/` directory
     of the LingPy package and automatically loaded when loading the LingPy
     library. The values are defined as the constants
-    :py:obj:`ipa_diacritics` and
-    :py:obj:`ipa_vowels`. Their core purpose is to guide the
+    :py:obj:`rcParams['vowels']`, :py:obj:`rcParams['diacritics']`, and
+    :py:obj:`rcParams['tones']`. Their core purpose is to guide the
     tokenization of IPA strings (cf.
-    :py:func:`~lingpy.algorithm.misc.ipa2tokens`). In order to change the
-    variables, one simply has to change the text files :file:`diacritics` and
+    :py:func:`~lingpy.sequence.sound_classes.ipa2tokens`). In order to change the
+    variables, one simply has to change the text files :file:`diacritics`,
+    :file:`tones`, and
     :file:`vowels` in the :file:`data/models/dv` directory. The structure of
     these files is fairly simple: Each line contains a vowel or a diacritic
     character, whereas diacritics are preceded by a dash.
