@@ -1,18 +1,29 @@
 # author   : Johann-Mattis List
 # email    : mattis.list@gmail.com
 # created  : 2013-03-06 23:14
-# modified : 2013-03-06 23:14
+# modified : 2013-10-02 20:31
 """
 Module for handling sequence models.
 """
 
 __author__="Johann-Mattis List"
-__date__="2013-03-06"
+__date__="2013-10-02"
 
 
-from re import findall
+import re
 from pickle import load
 import os
+import codecs
+
+from .._settings import rcParams
+try:
+    from .derive import compile_model, compile_dvt
+except ImportError:
+    print(rcParams['W_missing_module'].format("networkx"))
+
+from ..algorithm import misc
+from ..read import *
+from ..convert import *
 
 class Model(object):
     """
@@ -21,7 +32,7 @@ class Model(object):
     Parameters
     ----------
 
-    model : { 'sca', 'dolgo', 'asjp', 'art', 'color' }
+    model : { 'sca', 'dolgo', 'asjp', 'art', '_color' }
         A string indicating the name of the model which shall be loaded.
         Select between:
         
@@ -32,7 +43,7 @@ class Model(object):
           :evobib:`Brown2008` and :evobib:`Brown2011`),
         * 'art - the sound-class model which is used for the calculation of
           sonority profiles and prosodic strings (see :evobib:`List2012`), and
-        * 'color" - the sound-class model which is used for the coloring of
+        * '_color" - the sound-class model which is used for the coloring of
           sound-tokens when creating html-output.  
     
     Notes
@@ -67,10 +78,18 @@ class Model(object):
     Examples
     --------
     When loading LingPy, the models ``sca``, ``asjp``, ``dolgo``, and ``art``
-    are automatically loaded:
+    are automatically loaded, and they are accessible as global variables via
+    their names:
 
     >>> from lingpy import *
 
+    Define a couple of standard models for convenience.
+
+    >>> asjp = Model('asjp')
+    >>> sca = Model('sca')
+    >>> dolgo = Model('dolgo')
+    >>> art = model('art')
+
     Check, how the letter ``a`` is converted in the various models:
 
     >>> for m in [asjp,sca,dolgo,art]: 
@@ -94,44 +113,63 @@ class Model(object):
     See also
     --------
     lingpy.data.derive.compile_model
-    lingpy.data.derive.compile_diacritics_and_vowels
+    lingpy.data.derive.compile_dvt
 
     """
 
     def __init__(
             self,
             model,
-            path = None
+            path = ""
             ):
         
         if not path:
-            path = os.path.split(os.path.abspath(__file__))[0]+'/models/'+model+'/'
+            new_path = os.path.join(
+                    rcParams['_path'],
+                    'data',
+                    'models',
+                    model
+                    )
         else:
-            if path.endswith('/'):
-                path = path+model+'/'
-            else:
-                path = path + '/' + model + '/'
-
+            new_path = os.path.join(path,model)
         self.name = model
-        self.converter = load(open(path+'converter.bin','rb'))
-        try:
-            self.scorer = load(open(path+'scorer.bin','rb'))
-        except:
+        # check for converter
+        if not os.path.isfile(os.path.join(new_path,'converter.bin')):
+            compile_model(model,path)
+        
+        self.converter = load(open(os.path.join(new_path,'converter.bin'),'rb'))
+
+        # check for scorer
+        
+        # give always preference to scorer matrix files
+        if os.path.isfile(os.path.join(new_path,'matrix')):
+            self.scorer = read_scorer(os.path.join(new_path,'matrix'))
+        elif os.path.isfile(os.path.join(new_path,'scorer.bin')):
+            self.scorer = load(open(os.path.join(new_path,'scorer.bin'),'rb'))
+
+        # if none of the above fits, leave it
+        else: 
             pass
         
         # read information from the info-file
         self.info = {}
         
-        info = open(path+'INFO').read()
+        info = codecs.open(os.path.join(new_path,'INFO'),'r','utf-8').read()
         
-        data = ['description','compiler','source','date']
+        data = ['description','compiler','source','date','vowels','tones']
         
         for line in data:
             try:
-                self.info[line] = findall('@'+line+': (.*)',info)[0]
+                self.info[line] = re.findall('@'+line+': (.*)',info)[0]
             except:
                 self.info[line] = 'unknown'
 
+        # check for vowels and tones
+        if "vowels" in self.info:
+            self.vowels = self.info['vowels']
+        if "tones" in self.info:
+            self.tones = self.info['tones']
+
     def __str__(self):
         
         out = 'Model:    {0}\nInfo:     {1}\nSource:   {2}\n'
@@ -176,13 +214,33 @@ class Model(object):
         """
         return self.scorer[x,y]
 
-def load_dvt():
+def load_dvt(path=''):
     """
     Function loads the default characters for IPA diacritics and IPA vowels of LingPy.
     """
+    if not path:
+        pathx = os.path.join(
+                rcParams['_path'],
+                'data',
+                'models',
+                'dvt',
+                'dvt.bin'
+                )
+    elif path in ['el','evolaemp']:
+        pathx = os.path.join(
+                rcParams['_path'],
+                'data',
+                'models',
+                'dvt_el',
+                'dvt.bin'
+                )
+    else:
+        pass
+    
+    try:
+        dvt = load(open(pathx,'rb'))
+    except:
+        compile_dvt(path)
+        dvt = load(open(pathx,'rb'))
     
-    path = os.path.split(os.path.abspath(__file__))[0]+'/models/dvt/dvt.bin'
-
-    dvt = load(open(path,'rb'))
-
     return dvt
