@@ -1,28 +1,23 @@
 # author   : Johann-Mattis List
 # email    : mattis.list@gmail.com
 # created  : 2013-03-04 13:27
-# modified : 2013-04-06 12:25
+# modified : 2013-07-13 10:41
 """
 Module provides various methods for the handling of sound classes.
 
 """
 
 __author__="Johann-Mattis List"
-__date__="2013-04-06"
+__date__="2013-07-13"
 
 # lingpy imports
-from ..data import *
+from ..settings import rcParams
+#from ..data import *
 from ..data.ipa.sampa import reXS,xs
 
 def ipa2tokens(
         istring,
-        diacritics = None,
-        vowels = None,
-        tones = None,
-        combiners = '\u0361\u035c',
-        breaks = '.-',
-        stress = "ˈˌ'",
-        merge_vowels = True
+        **keywords
         ):
     """
     Tokenize IPA-encoded strings. 
@@ -81,6 +76,18 @@ def ipa2tokens(
     tokens2class
     class2tokens
     """
+    # go for defaults
+    kw = dict(
+            vowels = rcParams['vowels'],
+            diacritics = rcParams['diacritics'],
+            tones = rcParams['tones'],
+            combiners = rcParams['combiners'],
+            breaks = rcParams['breaks'],
+            stress = rcParams['stress'],
+            merge_vowels = rcParams['merge_vowels']
+            )
+    kw.update(keywords)
+
     # check for pre-tokenized strings
     if ' ' in istring:
         out = istring.split(' ')
@@ -88,14 +95,6 @@ def ipa2tokens(
             return out[1:-1]
         else:
             return out
-
-    # check for parameters
-    if not vowels:
-        vowels = ipa_vowels
-    if not tones:
-        tones = ipa_tones
-    if not diacritics:
-        diacritics = ipa_diacritics
     
     # create the list for the output
     out = []
@@ -108,21 +107,22 @@ def ipa2tokens(
 
     for char in istring:
         # check for breaks first, since they force us to start anew
-        if char in breaks:
+        if char in kw['breaks']:
             start = True
             vowel = False
             tone = False
             merge = False
         
         # check for combiners next
-        elif char in combiners:
+        elif char in kw['combiners']:
             out[-1] += char
             merge = True
 
         # check for stress
-        elif char in stress:
+        elif char in kw['stress']:
             out += [char]
-            start = True
+            # XXX be careful about the removement of the start-flag here, but it
+            # XXX seems to make sense so far!
             merge = True
             tone = False
             vowel = False
@@ -130,12 +130,12 @@ def ipa2tokens(
         # check for merge command 
         elif merge:
             out[-1] += char
-            if char in vowels:
+            if char in kw['vowels']:
                 vowel = True
             merge = False
         
         # check for diacritics
-        elif char in diacritics:
+        elif char in kw['diacritics']:
             if not start:
                 out[-1] += char
             else:
@@ -144,8 +144,8 @@ def ipa2tokens(
                 merge = True
         
         # check for vowels
-        elif char in vowels:
-            if vowel and merge_vowels:
+        elif char in kw['vowels']:
+            if vowel and kw['merge_vowels']:
                 out[-1] += char
             else:
                 out += [char]
@@ -154,7 +154,8 @@ def ipa2tokens(
             tone = False
         
         # check for tones
-        elif char in tones:
+        elif char in kw['tones']:
+            vowel = False
             if tone:
                 out[-1] += char
             else:
@@ -165,6 +166,7 @@ def ipa2tokens(
         # consonants
         else:
             vowel = False
+            tone = False
             out += [char]
             start = False
             tone = False
@@ -174,8 +176,7 @@ def ipa2tokens(
 def tokens2class(
         tstring,
         model,
-        stress = "ˈˌ'",
-
+        **keywords
         ):
     """
     Convert tokenized IPA strings into their respective class strings.
@@ -199,7 +200,7 @@ def tokens2class(
     --------
     >>> from lingpy import *
     >>> tokens = ipa2tokens('t͡sɔyɡə')
-    >>> classes = tokens2class(tokens,sca)
+    >>> classes = tokens2class(tokens,'sca')
     >>> print(classes)
     CUKE
 
@@ -209,6 +210,15 @@ def tokens2class(
     class2tokens
 
     """
+    kw = dict(
+            stress = rcParams['stress']
+            )
+    kw.update(keywords)
+    
+    # check whether model is passed as real model or as string
+    if str(model) == model:
+        model = rcParams[model]
+    
     out = []
     for token in tstring:
         try:
@@ -218,20 +228,28 @@ def tokens2class(
                 out.append(model[token[0]])
             except:
                 # check for stressed syllables
-                if token[0] in stress:
-                    try:
-                        out.append(model[token[1:]])
-                    except:
-                        out.append(model[token[1]])
+                if len(token) > 0:
+                    if token[0] in kw['stress']:
+                        try:
+                            out.append(model[token[1:]])
+                        except:
+                            try:
+                                out.append(model[token[1]])
+                            except:
+                                # new character for missing data and spurious items
+                                out.append('0')
+                    else:
+                        # new character for missing data and spurious items
+                        out.append('0')
                 else:
-                    # new character for missing data and spurious items
-                    out.append('0')
+                    raise ValueError("[!] string '{0}' is erroneously coded!".format(tstring))
 
     return out
 
 def prosodic_string(
         string,
-        _output = True
+        _output = True,
+        **keywords
         ):
     """
     Create a prosodic string of the sonority profile of a sequence.
@@ -277,11 +295,18 @@ def prosodic_string(
 
     Examples
     --------
-    >>> profile = [int(i) for i in tokens2class(ipa2tokens('t͡sɔyɡə'),art)]
+    >>> profile = [int(i) for i in tokens2class(ipa2tokens('t͡sɔyɡə'),'art')]
     >>> prosodic_string(profile)
     '#vC>'
 
     """
+    defaults = dict(
+            stress = rcParams['stress']
+            )
+    for k in defaults:
+        if k not in keywords:
+            keywords[k] = defaults[k]
+
     # check for empty string passed
     if not string:
         return ''
@@ -290,7 +315,11 @@ def prosodic_string(
     if type(string[0]) != int:
         
         # get the sonority profile
-        sstring = [9] + [int(t) for t in tokens2class(string,art)] + [9]
+        sstring = [9] + [int(t) for t in tokens2class(
+            string,
+            rcParams['art'],
+            **keywords)
+            ] + [9]
     else:
         sstring = [9] + string + [9]
 
@@ -415,7 +444,6 @@ def prosodic_string(
 
 def prosodic_weights(
         prostring,
-        factor = 0.3,
         _transform = {}
         ):
     """
@@ -426,15 +454,10 @@ def prosodic_weights(
 
     prostring : string
         A prosodic string as it is returned by :py:func:`prosodic_string`.
-
-    scale : tuple or list
-        A tuple or list of floats indicating the degree by which the gaps in
-        the environment of ascending, maximum, and descending sonority should be
-        decreased or increased.
-
-    factor : float
-        A scaling factor by which the specific positions of initial and final
-        should be increased and decreased.
+    _transform : dict
+        A dictionary that determines how prosodic strings should be transformed
+        into prosodic weights. Use this dictionary to adjust the prosodic
+        strings to your own user-defined prosodic weight schema.
 
     Returns
     -------
@@ -453,17 +476,12 @@ def prosodic_weights(
     >>> from lingpy import *
     >>> prostring = '#vC>'
     >>> prosodic_weights(prostring)
-    [1.5600000000000001, 1.0, 1.2, 0.69999999999999996]
-    >>> prosodic_weights(prostring,scale=(4,1,2),factor=0.5)
-    [6.0, 1, 4, 0.5]
+    [2.0, 1.3, 1.5, 0.7]
     
     See also
     --------
     prosodic_string
 
-    Todo
-    ----
-    Change the documentation for the new scaling mode!
     """
     # check for transformer
     if _transform:
@@ -743,23 +761,35 @@ def pid(
         
         return idn_pos / len(almA)
 
-def check_tokens(tokens):
+def check_tokens(tokens,**keywords):
     """
     Function checks whether tokens are given in a consistent input format.
     """
-    
+    defaults = dict(
+            stress = rcParams['stress']
+            )
+    for k in defaults:
+        if k not in keywords:
+            keywords[k] = defaults[k]
+
     errors = []
 
     for i,token in enumerate(tokens):
         
-        # check for conversion within the articulation-model
+        # check for conversion within the rcParams['art']iculation-model
         try:
-            art.converter[token]
-        except:
+            rcParams['art'].converter[token]
+        except KeyError:
             try:
-                art.converter[token[0]]
-            except:
-                errors.append((i,token))
+                rcParams['art'].converter[token[0]]
+            except KeyError:
+                if token[0] in keywords['stress']:
+                    try:
+                        rcParams['art'].converter[token[1]]
+                    except KeyError:
+                        errors.append((i,token))
+                else:
+                    errors.append((i,token))
     
     return errors
 
@@ -815,7 +845,13 @@ def get_all_ngrams(sequence):
 
 def sampa2uni(seq):
     """
-    Convert sequence in ipa.sampa-format to ipa.unicode.
+    Convert sequence in IPA-sampa-format to IPA-unicode.
+
+    Notes
+    -----
+    This function is based on code taken from Peter Kleiweg
+    (http://www.let.rug.nl/~kleiweg/L04/devel/python/xsampa.html). 
+
     """
 
     result = ''
