@@ -24,9 +24,11 @@ Definition of relevant terms or abbreviations:
        from a node
     -  stem: the edge immediately preceeding a clade
 """
+import sys
 from numpy import zeros, argsort
 from copy import deepcopy
 import re
+import codecs
 from .newick import parse_string as newick_parse_string
 #from cogent.util.transform import comb
 #from cogent.maths.stats.test import correlation
@@ -46,6 +48,12 @@ __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
 
+if sys.version_info[0] > 2:
+    from functools import reduce
+
+    def cmp(a, b):
+        return (a < b) - (b < a)
+
 
 def comb(items, n=None):
     """Yields each successive combination of n items.
@@ -100,6 +108,9 @@ class TreeNode(object):
         self._parent = Parent
         if (Parent is not None) and not (self in Parent.Children):
             Parent.append(self)
+        
+        # added taxa here, for convenience when using tree-class in lingpy JML
+        self.taxa = self.getTipNames()
 
 ### built-in methods and list interface support
     def __repr__(self):
@@ -623,16 +634,79 @@ class TreeNode(object):
             # increase black count, multiple children lead to here
             if curr:
                 curr.black.append(prev)
-
+                
         curr = self
         while len(curr.black) == 1:
-            curr = curr.black[0]
+            
+            # curr = curr.black[0]
+            # changed above statement in order to check for missing attribute
+            # in lower node JML
+            currX = curr.black[0]
+            if not hasattr(currX,'black'): return curr
+            else: curr = currX
+
+        # clear all black attributes from the tree, added by JML
+        for n,t in self.getNodesDict().items():
+            try:
+                delattr(t,'black')
+            except:
+                pass
 
         return curr
 
     lca = lastCommonAncestor #for convenience
     
     #support for more advanced tree operations
+    def get_LCA(self,*nodes):
+        """
+        Find lowest common ancestor of a given number of nodes.
+
+        Notes
+        -----
+        This function is supposed to yield the same output as
+        lowestCommonAncestor does. It was added in order to overcome certain
+        problems in the original function, resulting from attributes added to a
+        PhyloNode-object that make the use at time unsecure. Furthermore, it
+        works with an arbitrary list of nodes (including tips and internal
+        nodes).
+        """
+        # XXX function added by JML
+
+        # check for nodes that are not in the list of the nodes of self
+        if not set(nodes).issubset(set(self.getNodeNames())):
+            raise ValueError(
+                    "[i] There are nodes that do not occur on the tree."
+                )
+
+        # make a dictionary that stores which nodes have been visited
+        visited = set()
+
+        # pick one node at random (first one)
+        queue = [nodes[0]]
+        
+        while len(visited) < len(nodes):# sum(visited) < len(visited):
+            
+            # get nodes from queue
+            n = queue.pop(0)
+            node = self.getNodeMatchingName(n)
+            
+            # get all tips
+            tips = node.getNodeNames()
+
+            for t in tips:
+                if t in nodes:
+                    visited.add(t)
+            
+            # check for visited
+            if len(visited) == len(nodes):
+                return node
+
+            p = node.Parent.Name
+
+            # append parent to queue
+            queue += [p] 
+            
+        return self.getNodeMatchingName(n)
     
     def separation(self, other):
         """Returns number of edges separating self and other."""
@@ -1237,11 +1311,23 @@ class TreeNode(object):
         (score, tree) = self._sorted(full_sort_order)
         return tree
     
-    def _asciiArt(self, char1='-', show_internal=True, compact=False):
+    def _asciiArt(self, char1='-', show_internal=True, compact=False,
+            labels=False):
+        """
+        Notes
+        -----
+        Added a labels-keyword to this function. This is useful for debugging,
+        since it allows the user to replace all nodes with a specific label,
+        and print it to the tree, accordingly.
+        """
+        # XXX Added labels-keywords: JML
         LEN = 10
         PAD = ' ' * LEN
         PA = ' ' * (LEN-1)
-        namestr = self.Name or '' # prevents name of NoneType
+        if not labels:
+            namestr = self.Name or '' # prevents name of NoneType
+        else:
+            namestr = labels[self.Name]
         if self.Children:
             mids = []
             result = []
@@ -1252,7 +1338,8 @@ class TreeNode(object):
                     char2 = '\\'
                 else:
                     char2 = '-'
-                (clines, mid) = c._asciiArt(char2, show_internal, compact)
+                (clines, mid) = c._asciiArt(char2, show_internal, compact,
+                        labels)
                 mids.append(mid+len(result))
                 result.extend(clines)
                 if not compact:
@@ -1271,15 +1358,20 @@ class TreeNode(object):
         else:
             return ([char1 + '-' + namestr], 0)
     
-    def asciiArt(self, show_internal=True, compact=False):
+    def asciiArt(self, show_internal=True, compact=False, labels=False):
         """Returns a string containing an ascii drawing of the tree.
         
         Arguments:
         - show_internal: includes internal edge names.
         - compact: use exactly one line per tip.
+        - labels: specify specific labels for all nodes in the tree.
+
+        Notes:
+        The labels-keyword was added to the function by JML.
         """
+        # XXX added labels-keywords JML
         (lines, mid) = self._asciiArt(
-                show_internal=show_internal, compact=compact)
+                show_internal=show_internal, compact=compact, labels=labels)
         return '\n'.join(lines)
     
     def _getXmlLines(self, indent=0, parent_params=None):
@@ -1326,7 +1418,7 @@ class TreeNode(object):
             data = self.getXML()
         else:
             data = self.getNewick(with_distances=with_distances)
-        outf = open(filename, "w")
+        outf = codecs.open(filename, "w",'utf-8')
         outf.writelines(data)
         outf.close()
 
@@ -2126,8 +2218,7 @@ class TreeBuilder(object):
         self._known_edges[id(node)] = node
         return node
 
-def LoadTree(filename=None, treestring=None, tip_names=None, format=None, \
-    underscore_unmunge=False):
+def LoadTree(filename=None, treestring=None, tip_names=None, underscore_unmunge=False):
 
     """Constructor for tree.
     
@@ -2143,24 +2234,17 @@ def LoadTree(filename=None, treestring=None, tip_names=None, format=None, \
 
     if filename:
         assert not (treestring or tip_names)
-        treestring = open(filename).read()
-        if format is None and filename.endswith('.xml'):
-            format = "xml"
+        # slight modification for easy import of treestrings instead of
+        # file-reading by JML
+        if filename.endswith(';'):
+            treestring = filename
+        else:
+            treestring = codecs.open(filename,'r','utf-8').read()
     if treestring:
         assert not tip_names
-        if format is None and treestring.startswith('<'):
-            format = "xml"
-        if format == "xml":
-            parser = tree_xml_parse_string
-        else:
-            parser = newick_parse_string
         tree_builder = TreeBuilder().createEdge
         #FIXME: More general strategy for underscore_unmunge
-        if parser is newick_parse_string:
-            tree = parser(treestring, tree_builder, \
-                    underscore_unmunge=underscore_unmunge)
-        else:
-            tree = parser(treestring, tree_builder)
+        tree = newick_parse_string(treestring, tree_builder, underscore_unmunge=underscore_unmunge)
         if not tree.NameLoaded:
             tree.Name = 'root'
     elif tip_names:
